"""
Python module generated from Java source file org.bukkit.material.PistonExtensionMaterial

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import BlockFace
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class PistonExtensionMaterial(MaterialData, Attachable):
    """
    Material data for the piston extension block

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self, type: "Material"):
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def setFacingDirection(self, face: "BlockFace") -> None:
        ...


    def getFacing(self) -> "BlockFace":
        ...


    def isSticky(self) -> bool:
        """
        Checks if this piston extension is sticky, and returns True if so

        Returns
        - True if this piston is "sticky", or False
        """
        ...


    def setSticky(self, sticky: bool) -> None:
        """
        Sets whether or not this extension is sticky

        Arguments
        - sticky: True if sticky, otherwise False
        """
        ...


    def getAttachedFace(self) -> "BlockFace":
        ...


    def clone(self) -> "PistonExtensionMaterial":
        ...



"""
Python module generated from Java source file java.nio.file.FileSystems

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.lang.reflect import Constructor
from java.net import URI
from java.nio.file import *
from java.nio.file.spi import FileSystemProvider
from java.security import AccessController
from java.security import PrivilegedAction
from java.util import Collections
from java.util import ServiceConfigurationError
from java.util import ServiceLoader
from jdk.internal.misc import VM
from sun.nio.fs import DefaultFileSystemProvider
from typing import Any, Callable, Iterable, Tuple


class FileSystems:

    @staticmethod
    def getDefault() -> "FileSystem":
        """
        Returns the default `FileSystem`. The default file system creates
        objects that provide access to the file systems accessible to the Java
        virtual machine. The *working directory* of the file system is
        the current user directory, named by the system property `user.dir`.
        This allows for interoperability with the java.io.File java.io.File
        class.
        
         The first invocation of any of the methods defined by this class
        locates the default FileSystemProvider provider object. Where the
        system property `java.nio.file.spi.DefaultFileSystemProvider` is
        not defined then the default provider is a system-default provider that
        is invoked to create the default file system.
        
         If the system property `java.nio.file.spi.DefaultFileSystemProvider`
        is defined then it is taken to be a list of one or more fully-qualified
        names of concrete provider classes identified by the URI scheme
        `"file"`. Where the property is a list of more than one name then
        the names are separated by a comma. Each class is loaded, using the system
        class loader, and instantiated by invoking a one argument constructor
        whose formal parameter type is `FileSystemProvider`. The providers
        are loaded and instantiated in the order they are listed in the property.
        If this process fails or a provider's scheme is not equal to `"file"`
        then an unspecified error is thrown. URI schemes are normally compared
        without regard to case but for the default provider, the scheme is
        required to be `"file"`. The first provider class is instantiated
        by invoking it with a reference to the system-default provider.
        The second provider class is instantiated by invoking it with a reference
        to the first provider instance. The third provider class is instantiated
        by invoking it with a reference to the second instance, and so on. The
        last provider to be instantiated becomes the default provider; its `getFileSystem` method is invoked with the URI `"file:///"` to
        get a reference to the default file system.
        
         Subsequent invocations of this method return the file system that was
        returned by the first invocation.

        Returns
        - the default file system
        """
        ...


    @staticmethod
    def getFileSystem(uri: "URI") -> "FileSystem":
        """
        Returns a reference to an existing `FileSystem`.
        
         This method iterates over the FileSystemProvider.installedProviders()
        installed providers to locate the provider that is identified by the URI
        URI.getScheme scheme of the given URI. URI schemes are compared
        without regard to case. The exact form of the URI is highly provider
        dependent. If found, the provider's FileSystemProvider.getFileSystem
        getFileSystem method is invoked to obtain a reference to the `FileSystem`.
        
         Once a file system created by this provider is FileSystem.close
        closed it is provider-dependent if this method returns a reference to
        the closed file system or throws FileSystemNotFoundException.
        If the provider allows a new file system to be created with the same URI
        as a file system it previously created then this method throws the
        exception if invoked after the file system is closed (and before a new
        instance is created by the .newFileSystem newFileSystem method).
        
         If a security manager is installed then a provider implementation
        may require to check a permission before returning a reference to an
        existing file system. In the case of the FileSystems.getDefault
        default file system, no permission check is required.

        Arguments
        - uri: the URI to locate the file system

        Returns
        - the reference to the file system

        Raises
        - IllegalArgumentException: if the pre-conditions for the `uri` parameter are not met
        - FileSystemNotFoundException: if the file system, identified by the URI, does not exist
        - ProviderNotFoundException: if a provider supporting the URI scheme is not installed
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission
        """
        ...


    @staticmethod
    def newFileSystem(uri: "URI", env: dict[str, Any]) -> "FileSystem":
        """
        Constructs a new file system that is identified by a URI
        
         This method iterates over the FileSystemProvider.installedProviders()
        installed providers to locate the provider that is identified by the URI
        URI.getScheme scheme of the given URI. URI schemes are compared
        without regard to case. The exact form of the URI is highly provider
        dependent. If found, the provider's FileSystemProvider.newFileSystem(URI,Map)
        newFileSystem(URI,Map) method is invoked to construct the new file system.
        
         Once a file system is FileSystem.close closed it is
        provider-dependent if the provider allows a new file system to be created
        with the same URI as a file system it previously created.
        
         **Usage Example:**
        Suppose there is a provider identified by the scheme `"memory"`
        installed:
        ```
         FileSystem fs = FileSystems.newFileSystem(URI.create("memory:///?name=logfs"),
                                                   Map.of("capacity", "16G", "blockSize", "4k"));
        ```

        Arguments
        - uri: the URI identifying the file system
        - env: a map of provider specific properties to configure the file system;
                 may be empty

        Returns
        - a new file system

        Raises
        - IllegalArgumentException: if the pre-conditions for the `uri` parameter are not met,
                 or the `env` parameter does not contain properties required
                 by the provider, or a property value is invalid
        - FileSystemAlreadyExistsException: if the file system has already been created
        - ProviderNotFoundException: if a provider supporting the URI scheme is not installed
        - IOException: if an I/O error occurs creating the file system
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission required by the file system provider implementation
        """
        ...


    @staticmethod
    def newFileSystem(uri: "URI", env: dict[str, Any], loader: "ClassLoader") -> "FileSystem":
        """
        Constructs a new file system that is identified by a URI
        
         This method first attempts to locate an installed provider in exactly
        the same manner as the .newFileSystem(URI,Map) newFileSystem(URI,Map)
        method. If none of the installed providers support the URI scheme then an
        attempt is made to locate the provider using the given class loader. If a
        provider supporting the URI scheme is located then its FileSystemProvider.newFileSystem(URI,Map) newFileSystem(URI,Map) is
        invoked to construct the new file system.

        Arguments
        - uri: the URI identifying the file system
        - env: a map of provider specific properties to configure the file system;
                 may be empty
        - loader: the class loader to locate the provider or `null` to only
                 attempt to locate an installed provider

        Returns
        - a new file system

        Raises
        - IllegalArgumentException: if the pre-conditions for the `uri` parameter are not met,
                 or the `env` parameter does not contain properties required
                 by the provider, or a property value is invalid
        - FileSystemAlreadyExistsException: if the URI scheme identifies an installed provider and the file
                 system has already been created
        - ProviderNotFoundException: if a provider supporting the URI scheme is not found
        - ServiceConfigurationError: when an error occurs while loading a service provider
        - IOException: an I/O error occurs creating the file system
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission required by the file system provider implementation
        """
        ...


    @staticmethod
    def newFileSystem(path: "Path", loader: "ClassLoader") -> "FileSystem":
        """
        Constructs a new `FileSystem` to access the contents of a file as a
        file system.
        
         This method makes use of specialized providers that create pseudo file
        systems where the contents of one or more files is treated as a file
        system.
        
         This method first attempts to locate an installed provider in exactly
        the same manner as the .newFileSystem(Path, Map, ClassLoader)
        newFileSystem(Path, Map, ClassLoader) method with an empty map. If none
        of the installed providers return a `FileSystem` then an attempt is
        made to locate the provider using the given class loader. If a provider
        returns a file system then the lookup terminates and the file system is
        returned.

        Arguments
        - path: the path to the file
        - loader: the class loader to locate the provider or `null` to only
                 attempt to locate an installed provider

        Returns
        - a new file system

        Raises
        - ProviderNotFoundException: if a provider supporting this file type cannot be located
        - ServiceConfigurationError: when an error occurs while loading a service provider
        - IOException: if an I/O error occurs
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission
        """
        ...


    @staticmethod
    def newFileSystem(path: "Path", env: dict[str, Any]) -> "FileSystem":
        """
        Constructs a new `FileSystem` to access the contents of a file as a
        file system.
        
         This method makes use of specialized providers that create pseudo file
        systems where the contents of one or more files is treated as a file
        system.
        
         This method first attempts to locate an installed provider in exactly
        the same manner as the .newFileSystem(Path,Map,ClassLoader)
        newFileSystem(Path, Map, ClassLoader) method. If found, the provider's
        FileSystemProvider.newFileSystem(Path, Map) newFileSystem(Path, Map)
        method is invoked to construct the new file system.

        Arguments
        - path: the path to the file
        - env: a map of provider specific properties to configure the file system;
                 may be empty

        Returns
        - a new file system

        Raises
        - ProviderNotFoundException: if a provider supporting this file type cannot be located
        - ServiceConfigurationError: when an error occurs while loading a service provider
        - IOException: if an I/O error occurs
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission

        Since
        - 13
        """
        ...


    @staticmethod
    def newFileSystem(path: "Path") -> "FileSystem":
        """
        Constructs a new `FileSystem` to access the contents of a file as a
        file system.
        
         This method makes use of specialized providers that create pseudo file
        systems where the contents of one or more files is treated as a file
        system.
        
         This method first attempts to locate an installed provider in exactly
        the same manner as the .newFileSystem(Path,Map,ClassLoader)
        newFileSystem(Path, Map, ClassLoader) method. If found, the provider's
        FileSystemProvider.newFileSystem(Path, Map) newFileSystem(Path, Map)
        method is invoked with an empty map to construct the new file system.

        Arguments
        - path: the path to the file

        Returns
        - a new file system

        Raises
        - ProviderNotFoundException: if a provider supporting this file type cannot be located
        - ServiceConfigurationError: when an error occurs while loading a service provider
        - IOException: if an I/O error occurs
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission

        Since
        - 13
        """
        ...


    @staticmethod
    def newFileSystem(path: "Path", env: dict[str, Any], loader: "ClassLoader") -> "FileSystem":
        """
        Constructs a new `FileSystem` to access the contents of a file as a
        file system.
        
         This method makes use of specialized providers that create pseudo file
        systems where the contents of one or more files is treated as a file
        system.
        
         This method iterates over the FileSystemProvider.installedProviders()
        installed providers. It invokes, in turn, each provider's FileSystemProvider.newFileSystem(Path,Map) newFileSystem(Path,Map)
        method. If a provider returns a file system then the iteration
        terminates and the file system is returned.
        If none of the installed providers return a `FileSystem` then
        an attempt is made to locate the provider using the given class loader.
        If a provider returns a file
        system, then the lookup terminates and the file system is returned.

        Arguments
        - path: the path to the file
        - env: a map of provider specific properties to configure the file system;
                 may be empty
        - loader: the class loader to locate the provider or `null` to only
                 attempt to locate an installed provider

        Returns
        - a new file system

        Raises
        - ProviderNotFoundException: if a provider supporting this file type cannot be located
        - ServiceConfigurationError: when an error occurs while loading a service provider
        - IOException: if an I/O error occurs
        - SecurityException: if a security manager is installed and it denies an unspecified
                 permission

        Since
        - 13
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.Hasher

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.hash import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.nio.charset import Charset
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Hasher(PrimitiveSink):
    """
    A PrimitiveSink that can compute a hash code after reading the input. Each hasher should
    translate all multibyte values (.putInt(int), .putLong(long), etc) to bytes in
    little-endian order.
    
    **Warning:** The result of calling any methods after calling .hash is undefined.
    
    **Warning:** Using a specific character encoding when hashing a CharSequence with
    .putString(CharSequence, Charset) is generally only useful for cross-language
    compatibility (otherwise prefer .putUnencodedChars). However, the character encodings
    must be identical across languages. Also beware that Charset definitions may occasionally
    change between Java releases.
    
    **Warning:** Chunks of data that are put into the Hasher are not delimited. The
    resulting HashCode is dependent only on the bytes inserted, and the order in which they
    were inserted, not how those bytes were chunked into discrete put() operations. For example, the
    following three expressions all generate colliding hash codes:
    
    ````newHasher().putByte(b1).putByte(b2).putByte(b3).hash()
    newHasher().putByte(b1).putBytes(new byte[] { b2, b3`).hash()
    newHasher().putBytes(new byte[] { b1, b2, b3 }).hash()
    }```
    
    If you wish to avoid this, you should either prepend or append the size of each chunk. Keep in
    mind that when dealing with char sequences, the encoded form of two concatenated char sequences
    is not equivalent to the concatenation of their encoded form. Therefore, .putString(CharSequence, Charset) should only be used consistently with *complete*
    sequences and not broken into chunks.

    Author(s)
    - Kevin Bourrillion

    Since
    - 11.0
    """

    def putByte(self, b: int) -> "Hasher":
        ...


    def putBytes(self, bytes: list[int]) -> "Hasher":
        ...


    def putBytes(self, bytes: list[int], off: int, len: int) -> "Hasher":
        ...


    def putBytes(self, bytes: "ByteBuffer") -> "Hasher":
        ...


    def putShort(self, s: int) -> "Hasher":
        ...


    def putInt(self, i: int) -> "Hasher":
        ...


    def putLong(self, l: int) -> "Hasher":
        ...


    def putFloat(self, f: float) -> "Hasher":
        """
        Equivalent to `putInt(Float.floatToRawIntBits(f))`.
        """
        ...


    def putDouble(self, d: float) -> "Hasher":
        """
        Equivalent to `putLong(Double.doubleToRawLongBits(d))`.
        """
        ...


    def putBoolean(self, b: bool) -> "Hasher":
        """
        Equivalent to `putByte(b ? (byte) 1 : (byte) 0)`.
        """
        ...


    def putChar(self, c: str) -> "Hasher":
        ...


    def putUnencodedChars(self, charSequence: "CharSequence") -> "Hasher":
        """
        Equivalent to processing each `char` value in the `CharSequence`, in order. In
        other words, no character encoding is performed; the low byte and high byte of each `char` are hashed directly (in that order). The input must not be updated while this method is
        in progress.
        
        **Warning:** This method will produce different output than most other languages do when
        running the same hash function on the equivalent input. For cross-language compatibility, use
        .putString, usually with a charset of UTF-8. For other use cases, use `putUnencodedChars`.

        Since
        - 15.0 (since 11.0 as putString(CharSequence)).
        """
        ...


    def putString(self, charSequence: "CharSequence", charset: "Charset") -> "Hasher":
        """
        Equivalent to `putBytes(charSequence.toString().getBytes(charset))`.
        
        **Warning:** This method, which reencodes the input before hashing it, is useful only for
        cross-language compatibility. For other use cases, prefer .putUnencodedChars, which is
        faster, produces the same output across Java releases, and hashes every `char` in the
        input, even if some are invalid.
        """
        ...


    def putObject(self, instance: "T", funnel: "Funnel"["T"]) -> "Hasher":
        """
        A simple convenience for `funnel.funnel(object, this)`.
        """
        ...


    def hash(self) -> "HashCode":
        """
        Computes a hash code based on the data that have been provided to this hasher. The result is
        unspecified if this method is called more than once on the same instance.
        """
        ...


    def hashCode(self) -> int:
        """
        Deprecated
        - This returns Object.hashCode(); you almost certainly mean to call `hash().asInt()`.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.Ageable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import *
from typing import Any, Callable, Iterable, Tuple


class Ageable(BlockData):
    """
    'age' represents the different growth stages that a crop-like block can go
    through.
    
    A value of 0 indicates that the crop was freshly planted, whilst a value
    equal to .getMaximumAge() indicates that the crop is ripe and ready
    to be harvested.
    """

    def getAge(self) -> int:
        """
        Gets the value of the 'age' property.

        Returns
        - the 'age' value
        """
        ...


    def setAge(self, age: int) -> None:
        """
        Sets the value of the 'age' property.

        Arguments
        - age: the new 'age' value
        """
        ...


    def getMaximumAge(self) -> int:
        """
        Gets the maximum allowed value of the 'age' property.

        Returns
        - the maximum 'age' value
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.nodes.NodeTuple

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.nodes import *
from typing import Any, Callable, Iterable, Tuple


class NodeTuple:
    """
    Stores one key value pair used in a map.
    """

    def __init__(self, keyNode: "Node", valueNode: "Node"):
        ...


    def getKeyNode(self) -> "Node":
        """
        Key node.

        Returns
        - the node used as key
        """
        ...


    def getValueNode(self) -> "Node":
        """
        Value node.

        Returns
        - node used as value
        """
        ...


    def toString(self) -> str:
        ...



from .FoodComponent import FoodComponent
from .EquippableComponent import EquippableComponent
from .JukeboxPlayableComponent import JukeboxPlayableComponent
from .UseCooldownComponent import UseCooldownComponent
from .CustomModelDataComponent import CustomModelDataComponent
from .ToolComponent import ToolComponent


"""
Python module generated from Java source file java.util.NavigableMap

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class NavigableMap(SortedMap):
    """
    A SortedMap extended with navigation methods returning the
    closest matches for given search targets. Methods
    .lowerEntry, .floorEntry, .ceilingEntry,
    and .higherEntry return `Map.Entry` objects
    associated with keys respectively less than, less than or equal,
    greater than or equal, and greater than a given key, returning
    `null` if there is no such key.  Similarly, methods
    .lowerKey, .floorKey, .ceilingKey, and
    .higherKey return only the associated keys. All of these
    methods are designed for locating, not traversing entries.
    
    A `NavigableMap` may be accessed and traversed in either
    ascending or descending key order.  The .descendingMap
    method returns a view of the map with the senses of all relational
    and directional methods inverted. The performance of ascending
    operations and views is likely to be faster than that of descending
    ones.  Methods
    .subMap(Object, boolean, Object, boolean) subMap(K, boolean, K, boolean),
    .headMap(Object, boolean) headMap(K, boolean), and
    .tailMap(Object, boolean) tailMap(K, boolean)
    differ from the like-named `SortedMap` methods in accepting
    additional arguments describing whether lower and upper bounds are
    inclusive versus exclusive.  Submaps of any `NavigableMap`
    must implement the `NavigableMap` interface.
    
    This interface additionally defines methods .firstEntry,
    .pollFirstEntry, .lastEntry, and
    .pollLastEntry that return and/or remove the least and
    greatest mappings, if any exist, else returning `null`.
    
    Implementations of entry-returning methods are expected to
    return `Map.Entry` pairs representing snapshots of mappings
    at the time they were produced, and thus generally do *not*
    support the optional `Entry.setValue` method. Note however
    that it is possible to change mappings in the associated map using
    method `put`.
    
    Methods
    .subMap(Object, Object) subMap(K, K),
    .headMap(Object) headMap(K), and
    .tailMap(Object) tailMap(K)
    are specified to return `SortedMap` to allow existing
    implementations of `SortedMap` to be compatibly retrofitted to
    implement `NavigableMap`, but extensions and implementations
    of this interface are encouraged to override these methods to return
    `NavigableMap`.  Similarly,
    .keySet() can be overridden to return NavigableSet.
    
    This interface is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<K>`: the type of keys maintained by this map
    
    Type `<V>`: the type of mapped values

    Author(s)
    - Josh Bloch

    Since
    - 1.6
    """

    def lowerEntry(self, key: "K") -> "Map.Entry"["K", "V"]:
        """
        Returns a key-value mapping associated with the greatest key
        strictly less than the given key, or `null` if there is
        no such key.

        Arguments
        - key: the key

        Returns
        - an entry with the greatest key less than `key`,
                or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def lowerKey(self, key: "K") -> "K":
        """
        Returns the greatest key strictly less than the given key, or
        `null` if there is no such key.

        Arguments
        - key: the key

        Returns
        - the greatest key less than `key`,
                or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def floorEntry(self, key: "K") -> "Map.Entry"["K", "V"]:
        """
        Returns a key-value mapping associated with the greatest key
        less than or equal to the given key, or `null` if there
        is no such key.

        Arguments
        - key: the key

        Returns
        - an entry with the greatest key less than or equal to
                `key`, or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def floorKey(self, key: "K") -> "K":
        """
        Returns the greatest key less than or equal to the given key,
        or `null` if there is no such key.

        Arguments
        - key: the key

        Returns
        - the greatest key less than or equal to `key`,
                or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def ceilingEntry(self, key: "K") -> "Map.Entry"["K", "V"]:
        """
        Returns a key-value mapping associated with the least key
        greater than or equal to the given key, or `null` if
        there is no such key.

        Arguments
        - key: the key

        Returns
        - an entry with the least key greater than or equal to
                `key`, or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def ceilingKey(self, key: "K") -> "K":
        """
        Returns the least key greater than or equal to the given key,
        or `null` if there is no such key.

        Arguments
        - key: the key

        Returns
        - the least key greater than or equal to `key`,
                or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def higherEntry(self, key: "K") -> "Map.Entry"["K", "V"]:
        """
        Returns a key-value mapping associated with the least key
        strictly greater than the given key, or `null` if there
        is no such key.

        Arguments
        - key: the key

        Returns
        - an entry with the least key greater than `key`,
                or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def higherKey(self, key: "K") -> "K":
        """
        Returns the least key strictly greater than the given key, or
        `null` if there is no such key.

        Arguments
        - key: the key

        Returns
        - the least key greater than `key`,
                or `null` if there is no such key

        Raises
        - ClassCastException: if the specified key cannot be compared
                with the keys currently in the map
        - NullPointerException: if the specified key is null
                and this map does not permit null keys
        """
        ...


    def firstEntry(self) -> "Map.Entry"["K", "V"]:
        """
        Returns a key-value mapping associated with the least
        key in this map, or `null` if the map is empty.

        Returns
        - an entry with the least key,
                or `null` if this map is empty
        """
        ...


    def lastEntry(self) -> "Map.Entry"["K", "V"]:
        """
        Returns a key-value mapping associated with the greatest
        key in this map, or `null` if the map is empty.

        Returns
        - an entry with the greatest key,
                or `null` if this map is empty
        """
        ...


    def pollFirstEntry(self) -> "Map.Entry"["K", "V"]:
        """
        Removes and returns a key-value mapping associated with
        the least key in this map, or `null` if the map is empty.

        Returns
        - the removed first entry of this map,
                or `null` if this map is empty
        """
        ...


    def pollLastEntry(self) -> "Map.Entry"["K", "V"]:
        """
        Removes and returns a key-value mapping associated with
        the greatest key in this map, or `null` if the map is empty.

        Returns
        - the removed last entry of this map,
                or `null` if this map is empty
        """
        ...


    def descendingMap(self) -> "NavigableMap"["K", "V"]:
        """
        Returns a reverse order view of the mappings contained in this map.
        The descending map is backed by this map, so changes to the map are
        reflected in the descending map, and vice-versa.  If either map is
        modified while an iteration over a collection view of either map
        is in progress (except through the iterator's own `remove`
        operation), the results of the iteration are undefined.
        
        The returned map has an ordering equivalent to
        Collections.reverseOrder(Comparator) Collections.reverseOrder`(comparator())`.
        The expression `m.descendingMap().descendingMap()` returns a
        view of `m` essentially equivalent to `m`.

        Returns
        - a reverse order view of this map
        """
        ...


    def navigableKeySet(self) -> "NavigableSet"["K"]:
        """
        Returns a NavigableSet view of the keys contained in this map.
        The set's iterator returns the keys in ascending order.
        The set is backed by the map, so changes to the map are reflected in
        the set, and vice-versa.  If the map is modified while an iteration
        over the set is in progress (except through the iterator's own `remove` operation), the results of the iteration are undefined.  The
        set supports element removal, which removes the corresponding mapping
        from the map, via the `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear` operations.
        It does not support the `add` or `addAll` operations.

        Returns
        - a navigable set view of the keys in this map
        """
        ...


    def descendingKeySet(self) -> "NavigableSet"["K"]:
        """
        Returns a reverse order NavigableSet view of the keys contained in this map.
        The set's iterator returns the keys in descending order.
        The set is backed by the map, so changes to the map are reflected in
        the set, and vice-versa.  If the map is modified while an iteration
        over the set is in progress (except through the iterator's own `remove` operation), the results of the iteration are undefined.  The
        set supports element removal, which removes the corresponding mapping
        from the map, via the `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear` operations.
        It does not support the `add` or `addAll` operations.

        Returns
        - a reverse order navigable set view of the keys in this map
        """
        ...


    def subMap(self, fromKey: "K", fromInclusive: bool, toKey: "K", toInclusive: bool) -> "NavigableMap"["K", "V"]:
        """
        Returns a view of the portion of this map whose keys range from
        `fromKey` to `toKey`.  If `fromKey` and
        `toKey` are equal, the returned map is empty unless
        `fromInclusive` and `toInclusive` are both True.  The
        returned map is backed by this map, so changes in the returned map are
        reflected in this map, and vice-versa.  The returned map supports all
        optional map operations that this map supports.
        
        The returned map will throw an `IllegalArgumentException`
        on an attempt to insert a key outside of its range, or to construct a
        submap either of whose endpoints lie outside its range.

        Arguments
        - fromKey: low endpoint of the keys in the returned map
        - fromInclusive: `True` if the low endpoint
               is to be included in the returned view
        - toKey: high endpoint of the keys in the returned map
        - toInclusive: `True` if the high endpoint
               is to be included in the returned view

        Returns
        - a view of the portion of this map whose keys range from
                `fromKey` to `toKey`

        Raises
        - ClassCastException: if `fromKey` and `toKey`
                cannot be compared to one another using this map's comparator
                (or, if the map has no comparator, using natural ordering).
                Implementations may, but are not required to, throw this
                exception if `fromKey` or `toKey`
                cannot be compared to keys currently in the map.
        - NullPointerException: if `fromKey` or `toKey`
                is null and this map does not permit null keys
        - IllegalArgumentException: if `fromKey` is greater than
                `toKey`; or if this map itself has a restricted
                range, and `fromKey` or `toKey` lies
                outside the bounds of the range
        """
        ...


    def headMap(self, toKey: "K", inclusive: bool) -> "NavigableMap"["K", "V"]:
        """
        Returns a view of the portion of this map whose keys are less than (or
        equal to, if `inclusive` is True) `toKey`.  The returned
        map is backed by this map, so changes in the returned map are reflected
        in this map, and vice-versa.  The returned map supports all optional
        map operations that this map supports.
        
        The returned map will throw an `IllegalArgumentException`
        on an attempt to insert a key outside its range.

        Arguments
        - toKey: high endpoint of the keys in the returned map
        - inclusive: `True` if the high endpoint
               is to be included in the returned view

        Returns
        - a view of the portion of this map whose keys are less than
                (or equal to, if `inclusive` is True) `toKey`

        Raises
        - ClassCastException: if `toKey` is not compatible
                with this map's comparator (or, if the map has no comparator,
                if `toKey` does not implement Comparable).
                Implementations may, but are not required to, throw this
                exception if `toKey` cannot be compared to keys
                currently in the map.
        - NullPointerException: if `toKey` is null
                and this map does not permit null keys
        - IllegalArgumentException: if this map itself has a
                restricted range, and `toKey` lies outside the
                bounds of the range
        """
        ...


    def tailMap(self, fromKey: "K", inclusive: bool) -> "NavigableMap"["K", "V"]:
        """
        Returns a view of the portion of this map whose keys are greater than (or
        equal to, if `inclusive` is True) `fromKey`.  The returned
        map is backed by this map, so changes in the returned map are reflected
        in this map, and vice-versa.  The returned map supports all optional
        map operations that this map supports.
        
        The returned map will throw an `IllegalArgumentException`
        on an attempt to insert a key outside its range.

        Arguments
        - fromKey: low endpoint of the keys in the returned map
        - inclusive: `True` if the low endpoint
               is to be included in the returned view

        Returns
        - a view of the portion of this map whose keys are greater than
                (or equal to, if `inclusive` is True) `fromKey`

        Raises
        - ClassCastException: if `fromKey` is not compatible
                with this map's comparator (or, if the map has no comparator,
                if `fromKey` does not implement Comparable).
                Implementations may, but are not required to, throw this
                exception if `fromKey` cannot be compared to keys
                currently in the map.
        - NullPointerException: if `fromKey` is null
                and this map does not permit null keys
        - IllegalArgumentException: if this map itself has a
                restricted range, and `fromKey` lies outside the
                bounds of the range
        """
        ...


    def subMap(self, fromKey: "K", toKey: "K") -> "SortedMap"["K", "V"]:
        """
        
        
        Equivalent to `subMap(fromKey, True, toKey, False)`.

        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def headMap(self, toKey: "K") -> "SortedMap"["K", "V"]:
        """
        
        
        Equivalent to `headMap(toKey, False)`.

        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def tailMap(self, fromKey: "K") -> "SortedMap"["K", "V"]:
        """
        
        
        Equivalent to `tailMap(fromKey, True)`.

        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...



"""
Python module generated from Java source file com.google.common.io.CharStreams

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import Closeable
from java.io import EOFException
from java.io import IOException
from java.io import Reader
from java.io import Writer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class CharStreams:
    """
    Provides utility methods for working with character streams.
    
    Some of the methods in this class take arguments with a generic type of `Readable &
    Closeable`. A java.io.Reader implements both of those interfaces. Similarly for `Appendable & Closeable` and java.io.Writer.

    Author(s)
    - Colin Decker

    Since
    - 1.0
    """

    @staticmethod
    def copy(from: "Readable", to: "Appendable") -> int:
        """
        Copies all characters between the Readable and Appendable objects. Does not
        close or flush either object.

        Arguments
        - from: the object to read from
        - to: the object to write to

        Returns
        - the number of characters copied

        Raises
        - IOException: if an I/O error occurs
        """
        ...


    @staticmethod
    def toString(r: "Readable") -> str:
        """
        Reads all characters from a Readable object into a String. Does not close the
        `Readable`.

        Arguments
        - r: the object to read from

        Returns
        - a string containing all the characters

        Raises
        - IOException: if an I/O error occurs
        """
        ...


    @staticmethod
    def readLines(r: "Readable") -> list[str]:
        """
        Reads all of the lines from a Readable object. The lines do not include
        line-termination characters, but do include other leading and trailing whitespace.
        
        Does not close the `Readable`. If reading files or resources you should use the Files.readLines and Resources.readLines methods.

        Arguments
        - r: the object to read from

        Returns
        - a mutable List containing all the lines

        Raises
        - IOException: if an I/O error occurs
        """
        ...


    @staticmethod
    def readLines(readable: "Readable", processor: "LineProcessor"["T"]) -> "T":
        """
        Streams lines from a Readable object, stopping when the processor returns `False`
        or all lines have been read and returning the result produced by the processor. Does not close
        `readable`. Note that this method may not fully consume the contents of `readable`
        if the processor stops processing early.

        Raises
        - IOException: if an I/O error occurs

        Since
        - 14.0
        """
        ...


    @staticmethod
    def exhaust(readable: "Readable") -> int:
        """
        Reads and discards data from the given `Readable` until the end of the stream is reached.
        Returns the total number of chars read. Does not close the stream.

        Since
        - 20.0
        """
        ...


    @staticmethod
    def skipFully(reader: "Reader", n: int) -> None:
        """
        Discards `n` characters of data from the reader. This method will block until the full
        amount has been skipped. Does not close the reader.

        Arguments
        - reader: the reader to read from
        - n: the number of characters to skip

        Raises
        - EOFException: if this stream reaches the end before skipping all the characters
        - IOException: if an I/O error occurs
        """
        ...


    @staticmethod
    def nullWriter() -> "Writer":
        """
        Returns a Writer that simply discards written chars.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def asWriter(target: "Appendable") -> "Writer":
        """
        Returns a Writer that sends all output to the given Appendable target. Closing the
        writer will close the target if it is Closeable, and flushing the writer will flush the
        target if it is java.io.Flushable.

        Arguments
        - target: the object to which output will be sent

        Returns
        - a new Writer object, unless target is a Writer, in which case the target is returned
        """
        ...



"""
Python module generated from Java source file org.bukkit.projectiles.ProjectileSource

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Projectile
from org.bukkit.projectiles import *
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class ProjectileSource:
    """
    Represents a valid source of a projectile.
    """

    def launchProjectile(self, projectile: type["T"]) -> "T":
        """
        Launches a Projectile from the ProjectileSource.
        
        Type `<T>`: a projectile subclass

        Arguments
        - projectile: class of the projectile to launch

        Returns
        - the launched projectile
        """
        ...


    def launchProjectile(self, projectile: type["T"], velocity: "Vector") -> "T":
        """
        Launches a Projectile from the ProjectileSource with an
        initial velocity.
        
        Type `<T>`: a projectile subclass

        Arguments
        - projectile: class of the projectile to launch
        - velocity: the velocity with which to launch

        Returns
        - the launched projectile
        """
        ...



"""
Python module generated from Java source file com.google.common.primitives.Ints

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Converter
from com.google.common.primitives import *
from com.google.errorprone.annotations import InlineMe
from java.io import Serializable
from java.util import AbstractList
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import RandomAccess
from java.util import Spliterator
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Ints(IntsMethodsForWeb):
    """
    Static utility methods pertaining to `int` primitives, that are not already found in either
    Integer or Arrays.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/PrimitivesExplained">primitive utilities</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 1.0
    """

    BYTES = Integer.SIZE / Byte.SIZE
    """
    The number of bytes required to represent a primitive `int` value.
    
    **Java 8+ users:** use Integer.BYTES instead.
    """
    MAX_POWER_OF_TWO = 1 << (Integer.SIZE - 2)
    """
    The largest power of two that can be represented as an `int`.

    Since
    - 10.0
    """


    @staticmethod
    def hashCode(value: int) -> int:
        """
        Returns a hash code for `value`; equal to the result of invoking `((Integer)
        value).hashCode()`.
        
        **Java 8+ users:** use Integer.hashCode(int) instead.

        Arguments
        - value: a primitive `int` value

        Returns
        - a hash code for the value
        """
        ...


    @staticmethod
    def checkedCast(value: int) -> int:
        """
        Returns the `int` value that is equal to `value`, if possible.

        Arguments
        - value: any value in the range of the `int` type

        Returns
        - the `int` value that equals `value`

        Raises
        - IllegalArgumentException: if `value` is greater than Integer.MAX_VALUE or
            less than Integer.MIN_VALUE
        """
        ...


    @staticmethod
    def saturatedCast(value: int) -> int:
        """
        Returns the `int` nearest in value to `value`.

        Arguments
        - value: any `long` value

        Returns
        - the same value cast to `int` if it is in the range of the `int` type,
            Integer.MAX_VALUE if it is too large, or Integer.MIN_VALUE if it is too
            small
        """
        ...


    @staticmethod
    def compare(a: int, b: int) -> int:
        """
        Compares the two specified `int` values. The sign of the value returned is the same as
        that of `((Integer) a).compareTo(b)`.
        
        **Note:** this method is now unnecessary and should be treated as deprecated; use the
        equivalent Integer.compare method instead.

        Arguments
        - a: the first `int` to compare
        - b: the second `int` to compare

        Returns
        - a negative value if `a` is less than `b`; a positive value if `a` is
            greater than `b`; or zero if they are equal
        """
        ...


    @staticmethod
    def contains(array: list[int], target: int) -> bool:
        """
        Returns `True` if `target` is present as an element anywhere in `array`.

        Arguments
        - array: an array of `int` values, possibly empty
        - target: a primitive `int` value

        Returns
        - `True` if `array[i] == target` for some value of `i`
        """
        ...


    @staticmethod
    def indexOf(array: list[int], target: int) -> int:
        """
        Returns the index of the first appearance of the value `target` in `array`.

        Arguments
        - array: an array of `int` values, possibly empty
        - target: a primitive `int` value

        Returns
        - the least index `i` for which `array[i] == target`, or `-1` if no
            such index exists.
        """
        ...


    @staticmethod
    def indexOf(array: list[int], target: list[int]) -> int:
        """
        Returns the start position of the first occurrence of the specified `target` within
        `array`, or `-1` if there is no such occurrence.
        
        More formally, returns the lowest index `i` such that `Arrays.copyOfRange(array,
        i, i + target.length)` contains exactly the same elements as `target`.

        Arguments
        - array: the array to search for the sequence `target`
        - target: the array to search for as a sub-sequence of `array`
        """
        ...


    @staticmethod
    def lastIndexOf(array: list[int], target: int) -> int:
        """
        Returns the index of the last appearance of the value `target` in `array`.

        Arguments
        - array: an array of `int` values, possibly empty
        - target: a primitive `int` value

        Returns
        - the greatest index `i` for which `array[i] == target`, or `-1` if no
            such index exists.
        """
        ...


    @staticmethod
    def min(*array: Tuple[int, ...]) -> int:
        """
        Returns the least value present in `array`.

        Arguments
        - array: a *nonempty* array of `int` values

        Returns
        - the value present in `array` that is less than or equal to every other value in
            the array

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def max(*array: Tuple[int, ...]) -> int:
        """
        Returns the greatest value present in `array`.

        Arguments
        - array: a *nonempty* array of `int` values

        Returns
        - the value present in `array` that is greater than or equal to every other value
            in the array

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def constrainToRange(value: int, min: int, max: int) -> int:
        """
        Returns the value nearest to `value` which is within the closed range `[min..max]`.
        
        If `value` is within the range `[min..max]`, `value` is returned
        unchanged. If `value` is less than `min`, `min` is returned, and if `value` is greater than `max`, `max` is returned.

        Arguments
        - value: the `int` value to constrain
        - min: the lower bound (inclusive) of the range to constrain `value` to
        - max: the upper bound (inclusive) of the range to constrain `value` to

        Raises
        - IllegalArgumentException: if `min > max`

        Since
        - 21.0
        """
        ...


    @staticmethod
    def concat(*arrays: Tuple[list[int], ...]) -> list[int]:
        """
        Returns the values from each provided array combined into a single array. For example, `concat(new int[] {a, b`, new int[] {}, new int[] {c}} returns the array `{a, b, c`}.

        Arguments
        - arrays: zero or more `int` arrays

        Returns
        - a single array containing all the values from the source arrays, in order

        Raises
        - IllegalArgumentException: if the total number of elements in `arrays` does not fit
            in an `int`
        """
        ...


    @staticmethod
    def toByteArray(value: int) -> list[int]:
        """
        Returns a big-endian representation of `value` in a 4-element byte array; equivalent to
        `ByteBuffer.allocate(4).putInt(value).array()`. For example, the input value `0x12131415` would yield the byte array `{0x12, 0x13, 0x14, 0x15`}.
        
        If you need to convert and concatenate several values (possibly even of different types),
        use a shared java.nio.ByteBuffer instance, or use com.google.common.io.ByteStreams.newDataOutput() to get a growable buffer.
        """
        ...


    @staticmethod
    def fromByteArray(bytes: list[int]) -> int:
        """
        Returns the `int` value whose big-endian representation is stored in the first 4 bytes of
        `bytes`; equivalent to `ByteBuffer.wrap(bytes).getInt()`. For example, the input
        byte array `{0x12, 0x13, 0x14, 0x15, 0x33`} would yield the `int` value `0x12131415`.
        
        Arguably, it's preferable to use java.nio.ByteBuffer; that library exposes much more
        flexibility at little cost in readability.

        Raises
        - IllegalArgumentException: if `bytes` has fewer than 4 elements
        """
        ...


    @staticmethod
    def fromBytes(b1: int, b2: int, b3: int, b4: int) -> int:
        """
        Returns the `int` value whose byte representation is the given 4 bytes, in big-endian
        order; equivalent to `Ints.fromByteArray(new byte[] {b1, b2, b3, b4`)}.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def stringConverter() -> "Converter"[str, "Integer"]:
        """
        Returns a serializable converter object that converts between strings and integers using Integer.decode and Integer.toString(). The returned converter throws NumberFormatException if the input string is invalid.
        
        **Warning:** please see Integer.decode to understand exactly how strings are
        parsed. For example, the string `"0123"` is treated as *octal* and converted to the
        value `83`.

        Since
        - 16.0
        """
        ...


    @staticmethod
    def ensureCapacity(array: list[int], minLength: int, padding: int) -> list[int]:
        """
        Returns an array containing the same values as `array`, but guaranteed to be of a
        specified minimum length. If `array` already has a length of at least `minLength`,
        it is returned directly. Otherwise, a new array of size `minLength + padding` is
        returned, containing the values of `array`, and zeroes in the remaining places.

        Arguments
        - array: the source array
        - minLength: the minimum length the returned array must guarantee
        - padding: an extra amount to "grow" the array by if growth is necessary

        Returns
        - an array containing the values of `array`, with guaranteed minimum length `minLength`

        Raises
        - IllegalArgumentException: if `minLength` or `padding` is negative
        """
        ...


    @staticmethod
    def join(separator: str, *array: Tuple[int, ...]) -> str:
        """
        Returns a string containing the supplied `int` values separated by `separator`. For
        example, `join("-", 1, 2, 3)` returns the string `"1-2-3"`.

        Arguments
        - separator: the text that should appear between consecutive values in the resulting string
            (but not at the start or end)
        - array: an array of `int` values, possibly empty
        """
        ...


    @staticmethod
    def lexicographicalComparator() -> "Comparator"[list[int]]:
        """
        Returns a comparator that compares two `int` arrays <a
        href="http://en.wikipedia.org/wiki/Lexicographical_order">lexicographically</a>. That is, it
        compares, using .compare(int, int)), the first pair of values that follow any common
        prefix, or when one array is a prefix of the other, treats the shorter array as the lesser. For
        example, `[] < [1] < [1, 2] < [2]`.
        
        The returned comparator is inconsistent with Object.equals(Object) (since arrays
        support only identity equality), but it is consistent with Arrays.equals(int[], int[]).

        Since
        - 2.0
        """
        ...


    @staticmethod
    def sortDescending(array: list[int]) -> None:
        """
        Sorts the elements of `array` in descending order.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def sortDescending(array: list[int], fromIndex: int, toIndex: int) -> None:
        """
        Sorts the elements of `array` between `fromIndex` inclusive and `toIndex`
        exclusive in descending order.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def reverse(array: list[int]) -> None:
        """
        Reverses the elements of `array`. This is equivalent to `Collections.reverse(Ints.asList(array))`, but is likely to be more efficient.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def reverse(array: list[int], fromIndex: int, toIndex: int) -> None:
        """
        Reverses the elements of `array` between `fromIndex` inclusive and `toIndex`
        exclusive. This is equivalent to `Collections.reverse(Ints.asList(array).subList(fromIndex, toIndex))`, but is likely to be more
        efficient.

        Raises
        - IndexOutOfBoundsException: if `fromIndex < 0`, `toIndex > array.length`, or
            `toIndex > fromIndex`

        Since
        - 23.1
        """
        ...


    @staticmethod
    def rotate(array: list[int], distance: int) -> None:
        """
        Performs a right rotation of `array` of "distance" places, so that the first element is
        moved to index "distance", and the element at index `i` ends up at index `(distance
        + i) mod array.length`. This is equivalent to `Collections.rotate(Ints.asList(array),
        distance)`, but is considerably faster and avoids allocation and garbage collection.
        
        The provided "distance" may be negative, which will rotate left.

        Since
        - 32.0.0
        """
        ...


    @staticmethod
    def rotate(array: list[int], distance: int, fromIndex: int, toIndex: int) -> None:
        """
        Performs a right rotation of `array` between `fromIndex` inclusive and `toIndex` exclusive. This is equivalent to `Collections.rotate(Ints.asList(array).subList(fromIndex, toIndex), distance)`, but is
        considerably faster and avoids allocations and garbage collection.
        
        The provided "distance" may be negative, which will rotate left.

        Raises
        - IndexOutOfBoundsException: if `fromIndex < 0`, `toIndex > array.length`, or
            `toIndex > fromIndex`

        Since
        - 32.0.0
        """
        ...


    @staticmethod
    def toArray(collection: Iterable["Number"]) -> list[int]:
        """
        Returns an array containing each value of `collection`, converted to a `int` value
        in the manner of Number.intValue.
        
        Elements are copied from the argument collection as if by `collection.toArray()`.
        Calling this method is as thread-safe as calling that method.

        Arguments
        - collection: a collection of `Number` instances

        Returns
        - an array containing the same values as `collection`, in the same order, converted
            to primitives

        Raises
        - NullPointerException: if `collection` or any of its elements is null

        Since
        - 1.0 (parameter was `Collection<Integer>` before 12.0)
        """
        ...


    @staticmethod
    def asList(*backingArray: Tuple[int, ...]) -> list["Integer"]:
        """
        Returns a fixed-size list backed by the specified array, similar to Arrays.asList(Object[]). The list supports List.set(int, Object), but any attempt to
        set a value to `null` will result in a NullPointerException.
        
        The returned list maintains the values, but not the identities, of `Integer` objects
        written to or read from it. For example, whether `list.get(0) == list.get(0)` is True for
        the returned list is unspecified.
        
        The returned list is serializable.
        
        **Note:** when possible, you should represent your data as an ImmutableIntArray
        instead, which has an ImmutableIntArray.asList asList view.

        Arguments
        - backingArray: the array to back the list

        Returns
        - a list view of the array
        """
        ...


    @staticmethod
    def tryParse(string: str) -> "Integer":
        """
        Parses the specified string as a signed decimal integer value. The ASCII character `'-'`
        (`'&#92;u002D'`) is recognized as the minus sign.
        
        Unlike Integer.parseInt(String), this method returns `null` instead of
        throwing an exception if parsing fails. Additionally, this method only accepts ASCII digits,
        and returns `null` if non-ASCII digits are present in the string.
        
        Note that strings prefixed with ASCII `'+'` are rejected, even though Integer.parseInt(String) accepts them.

        Arguments
        - string: the string representation of an integer value

        Returns
        - the integer value represented by `string`, or `null` if `string` has
            a length of zero or cannot be parsed as an integer value

        Raises
        - NullPointerException: if `string` is `null`

        Since
        - 11.0
        """
        ...


    @staticmethod
    def tryParse(string: str, radix: int) -> "Integer":
        """
        Parses the specified string as a signed integer value using the specified radix. The ASCII
        character `'-'` (`'&#92;u002D'`) is recognized as the minus sign.
        
        Unlike Integer.parseInt(String, int), this method returns `null` instead of
        throwing an exception if parsing fails. Additionally, this method only accepts ASCII digits,
        and returns `null` if non-ASCII digits are present in the string.
        
        Note that strings prefixed with ASCII `'+'` are rejected, even though Integer.parseInt(String) accepts them.

        Arguments
        - string: the string representation of an integer value
        - radix: the radix to use when parsing

        Returns
        - the integer value represented by `string` using `radix`, or `null` if
            `string` has a length of zero or cannot be parsed as an integer value

        Raises
        - IllegalArgumentException: if `radix < Character.MIN_RADIX` or `radix >
            Character.MAX_RADIX`
        - NullPointerException: if `string` is `null`

        Since
        - 19.0
        """
        ...



"""
Python module generated from Java source file com.google.gson.annotations.SerializedName

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.annotations import *
from typing import Any, Callable, Iterable, Tuple


class SerializedName:
    """
    An annotation that indicates this member should be serialized to JSON with the provided name
    value as its field name.
    
    This annotation will override any com.google.gson.FieldNamingPolicy, including the
    default field naming policy, that may have been set on the com.google.gson.Gson instance.
    A different naming policy can set using the `GsonBuilder` class. See com.google.gson.GsonBuilder.setFieldNamingPolicy(com.google.gson.FieldNamingPolicy) for more
    information.
    
    Here is an example of how this annotation is meant to be used:
    
    ```
    public class MyClass {
      &#64;SerializedName("name") String a;
      &#64;SerializedName(value="name1", alternate={"name2", "name3"}) String b;
      String c;
    
      public MyClass(String a, String b, String c) {
        this.a = a;
        this.b = b;
        this.c = c;
      }
    }
    ```
    
    The following shows the output that is generated when serializing an instance of the above
    example class:
    
    ```
    MyClass target = new MyClass("v1", "v2", "v3");
    Gson gson = new Gson();
    String json = gson.toJson(target);
    System.out.println(json);
    
    ===== OUTPUT =====
    {"name":"v1","name1":"v2","c":"v3"}
    ```
    
    NOTE: The value you specify in this annotation must be a valid JSON field name. While
    deserializing, all values specified in the annotation will be deserialized into the field. For
    example:
    
    ```
      MyClass target = gson.fromJson("{'name1':'v1'}", MyClass.class);
      assertEquals("v1", target.b);
      target = gson.fromJson("{'name2':'v2'}", MyClass.class);
      assertEquals("v2", target.b);
      target = gson.fromJson("{'name3':'v3'}", MyClass.class);
      assertEquals("v3", target.b);
    ```
    
    Note that MyClass.b is now deserialized from either name1, name2 or name3.

    Author(s)
    - Joel Leitch

    See
    - com.google.gson.FieldNamingPolicy
    """

    def value(self) -> str:
        """
        The desired name of the field when it is serialized or deserialized.

        Returns
        - the desired name of the field when it is serialized or deserialized
        """
        ...


    def alternate(self) -> list[str]:
        """
        The alternative names of the field when it is deserialized

        Returns
        - the alternative names of the field when it is deserialized
        """
        return {}



"""
Python module generated from Java source file org.bukkit.spawner.BaseSpawner

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import CreatureSpawner
from org.bukkit.block.spawner import SpawnRule
from org.bukkit.block.spawner import SpawnerEntry
from org.bukkit.entity import EntitySnapshot
from org.bukkit.entity import EntityType
from org.bukkit.entity.minecart import SpawnerMinecart
from org.bukkit.spawner import *
from typing import Any, Callable, Iterable, Tuple


class BaseSpawner:
    """
    Represents a basic entity spawner. 
    May be a SpawnerMinecart, CreatureSpawner or TrialSpawnerConfiguration.
    """

    def getSpawnedType(self) -> "EntityType":
        """
        Get the spawner's creature type.

        Returns
        - The creature type or null if it not set.
        """
        ...


    def setSpawnedType(self, creatureType: "EntityType") -> None:
        """
        Set the spawner's creature type. 
        This will override any entities that have been added with .addPotentialSpawn

        Arguments
        - creatureType: The creature type or null to clear.
        """
        ...


    def getDelay(self) -> int:
        """
        Get the spawner's delay.
        
        This is the delay, in ticks, until the spawner will spawn its next mob.

        Returns
        - The delay.
        """
        ...


    def setDelay(self, delay: int) -> None:
        """
        Set the spawner's delay.

        Arguments
        - delay: The delay.
        """
        ...


    def getRequiredPlayerRange(self) -> int:
        """
        Get the maximum distance a player can be in order for this
        spawner to be active.
        
        If this value is less than or equal to 0, this spawner is always active
        (given that there are players online).
        
        Default value is 16.

        Returns
        - the maximum distance a player can be in order for this
        spawner to be active.
        """
        ...


    def setRequiredPlayerRange(self, requiredPlayerRange: int) -> None:
        """
        Set the maximum distance a player can be in order for this
        spawner to be active.
        
        Setting this value to less than or equal to 0 will make this spawner
        always active (given that there are players online).

        Arguments
        - requiredPlayerRange: the maximum distance a player can be
        in order for this spawner to be active.
        """
        ...


    def getSpawnRange(self) -> int:
        """
        Get the radius around which the spawner will attempt to spawn mobs in.
        
        This area is square, includes the block the spawner is in, and is
        centered on the spawner's x,z coordinates - not the spawner itself.
        
        It is 2 blocks high, centered on the spawner's y-coordinate (its bottom);
        thus allowing mobs to spawn as high as its top surface and as low
        as 1 block below its bottom surface.
        
        Default value is 4.

        Returns
        - the spawn range
        """
        ...


    def setSpawnRange(self, spawnRange: int) -> None:
        """
        Set the new spawn range.

        Arguments
        - spawnRange: the new spawn range

        See
        - .getSpawnRange()
        """
        ...


    def getSpawnedEntity(self) -> "EntitySnapshot":
        """
        Gets the EntitySnapshot that will be spawned by this spawner or null
        if no entities have been assigned to this spawner. 
        
        All applicable data from the spawner will be copied, such as custom name,
        health, and velocity. 

        Returns
        - the entity snapshot or null if no entities have been assigned to this
                spawner.
        """
        ...


    def setSpawnedEntity(self, snapshot: "EntitySnapshot") -> None:
        """
        Sets the entity that will be spawned by this spawner. 
        This will override any previous entries that have been added with
        .addPotentialSpawn
        
        All applicable data from the snapshot will be copied, such as custom name,
        health, and velocity. 

        Arguments
        - snapshot: the entity snapshot or null to clear
        """
        ...


    def setSpawnedEntity(self, spawnerEntry: "SpawnerEntry") -> None:
        """
        Sets the SpawnerEntry that will be spawned by this spawner. 
        This will override any previous entries that have been added with
        .addPotentialSpawn

        Arguments
        - spawnerEntry: the spawner entry to use
        """
        ...


    def addPotentialSpawn(self, snapshot: "EntitySnapshot", weight: int, spawnRule: "SpawnRule") -> None:
        """
        Adds a new EntitySnapshot to the list of entities this spawner can
        spawn.
        
        The weight will determine how often this entry is chosen to spawn, higher
        weighted entries will spawn more often than lower weighted ones. 
        The SpawnRule will determine under what conditions this entry can
        spawn, passing null will use the default conditions for the given entity.

        Arguments
        - snapshot: the snapshot that will be spawned
        - weight: the weight
        - spawnRule: the spawn rule for this entity, or null
        """
        ...


    def addPotentialSpawn(self, spawnerEntry: "SpawnerEntry") -> None:
        """
        Adds a new SpawnerEntry to the list of entities this spawner can
        spawn.

        Arguments
        - spawnerEntry: the spawner entry to use

        See
        - .addPotentialSpawn(EntitySnapshot, int, SpawnRule)
        """
        ...


    def setPotentialSpawns(self, entries: Iterable["SpawnerEntry"]) -> None:
        """
        Sets the list of SpawnerEntry this spawner can spawn. 
        This will override any previous entries added with
        .addPotentialSpawn

        Arguments
        - entries: the list of entries
        """
        ...


    def getPotentialSpawns(self) -> list["SpawnerEntry"]:
        """
        Gets a list of potential spawns from this spawner or an empty list if no
        entities have been assigned to this spawner. 
        Changes made to the returned list will not be reflected in the spawner unless
        applied with .setPotentialSpawns

        Returns
        - a list of potential spawns from this spawner, or an empty list if no
                entities have been assigned to this spawner

        See
        - .getSpawnedType()
        """
        ...



"""
Python module generated from Java source file org.bukkit.attribute.AttributeModifier

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from java.nio.charset import StandardCharsets
from java.util import Locale
from java.util import Objects
from java.util import UUID
from java.util.regex import Pattern
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit.attribute import *
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import EquipmentSlotGroup
from org.bukkit.util import NumberConversions
from typing import Any, Callable, Iterable, Tuple


class AttributeModifier(ConfigurationSerializable, Keyed):
    """
    Concrete implementation of an attribute modifier.
    """

    def __init__(self, name: str, amount: float, operation: "Operation"):
        ...


    def __init__(self, uuid: "UUID", name: str, amount: float, operation: "Operation"):
        ...


    def __init__(self, uuid: "UUID", name: str, amount: float, operation: "Operation", slot: "EquipmentSlot"):
        ...


    def __init__(self, uuid: "UUID", name: str, amount: float, operation: "Operation", slot: "EquipmentSlotGroup"):
        ...


    def __init__(self, key: "NamespacedKey", amount: float, operation: "Operation", slot: "EquipmentSlotGroup"):
        ...


    def getUniqueId(self) -> "UUID":
        """
        Get the unique ID for this modifier.

        Returns
        - unique id

        See
        - .getKey()

        Deprecated
        - attributes are now identified by keys
        """
        ...


    def getKey(self) -> "NamespacedKey":
        ...


    def getName(self) -> str:
        """
        Get the name of this modifier.

        Returns
        - name
        """
        ...


    def getAmount(self) -> float:
        """
        Get the amount by which this modifier will apply its Operation.

        Returns
        - modification amount
        """
        ...


    def getOperation(self) -> "Operation":
        """
        Get the operation this modifier will apply.

        Returns
        - operation
        """
        ...


    def getSlot(self) -> "EquipmentSlot":
        """
        Get the EquipmentSlot this AttributeModifier is active on,
        or null if this modifier is applicable for any slot.

        Returns
        - the slot

        Deprecated
        - use .getSlotGroup()
        """
        ...


    def getSlotGroup(self) -> "EquipmentSlotGroup":
        """
        Get the EquipmentSlot this AttributeModifier is active on,
        or null if this modifier is applicable for any slot.

        Returns
        - the slot
        """
        ...


    def serialize(self) -> dict[str, "Object"]:
        ...


    def equals(self, other: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...


    @staticmethod
    def deserialize(args: dict[str, "Object"]) -> "AttributeModifier":
        ...


    class Operation(Enum):
        """
        Enumerable operation to be applied.
        """

        ADD_NUMBER = 0
        """
        Adds (or subtracts) the specified amount to the base value.
        """
        ADD_SCALAR = 1
        """
        Adds this scalar of amount to the base value.
        """
        MULTIPLY_SCALAR_1 = 2
        """
        Multiply amount by this value, after adding 1 to it.
        """



"""
Python module generated from Java source file org.bukkit.map.MapView

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import World
from org.bukkit.inventory.meta import MapMeta
from org.bukkit.map import *
from typing import Any, Callable, Iterable, Tuple


class MapView:
    """
    Represents a map item.
    """

    def getId(self) -> int:
        """
        Get the ID of this map item for use with MapMeta.

        Returns
        - The ID of the map.
        """
        ...


    def isVirtual(self) -> bool:
        """
        Check whether this map is virtual. A map is virtual if its lowermost
        MapRenderer is plugin-provided.

        Returns
        - Whether the map is virtual.
        """
        ...


    def getScale(self) -> "Scale":
        """
        Get the scale of this map.

        Returns
        - The scale of the map.
        """
        ...


    def setScale(self, scale: "Scale") -> None:
        """
        Set the scale of this map.

        Arguments
        - scale: The scale to set.
        """
        ...


    def getCenterX(self) -> int:
        """
        Get the center X position of this map.

        Returns
        - The center X position.
        """
        ...


    def getCenterZ(self) -> int:
        """
        Get the center Z position of this map.

        Returns
        - The center Z position.
        """
        ...


    def setCenterX(self, x: int) -> None:
        """
        Set the center X position of this map.

        Arguments
        - x: The center X position.
        """
        ...


    def setCenterZ(self, z: int) -> None:
        """
        Set the center Z position of this map.

        Arguments
        - z: The center Z position.
        """
        ...


    def getWorld(self) -> "World":
        """
        Get the world that this map is associated with. Primarily used by the
        internal renderer, but may be used by external renderers. May return
        null if the world the map is associated with is not loaded.

        Returns
        - The World this map is associated with.
        """
        ...


    def setWorld(self, world: "World") -> None:
        """
        Set the world that this map is associated with. The world is used by
        the internal renderer, and may also be used by external renderers.

        Arguments
        - world: The World to associate this map with.
        """
        ...


    def getRenderers(self) -> list["MapRenderer"]:
        """
        Get a list of MapRenderers currently in effect.

        Returns
        - A `List<MapRenderer>` containing each map renderer.
        """
        ...


    def addRenderer(self, renderer: "MapRenderer") -> None:
        """
        Add a renderer to this map.

        Arguments
        - renderer: The MapRenderer to add.
        """
        ...


    def removeRenderer(self, renderer: "MapRenderer") -> bool:
        """
        Remove a renderer from this map.

        Arguments
        - renderer: The MapRenderer to remove.

        Returns
        - True if the renderer was successfully removed.
        """
        ...


    def isTrackingPosition(self) -> bool:
        """
        Gets whether a position cursor should be shown when the map is near its
        center.

        Returns
        - tracking status
        """
        ...


    def setTrackingPosition(self, trackingPosition: bool) -> None:
        """
        Sets whether a position cursor should be shown when the map is near its
        center.

        Arguments
        - trackingPosition: tracking status
        """
        ...


    def isUnlimitedTracking(self) -> bool:
        """
        Whether the map will show a smaller position cursor (True), or no
        position cursor (False) when cursor is outside of map's range.

        Returns
        - unlimited tracking state
        """
        ...


    def setUnlimitedTracking(self, unlimited: bool) -> None:
        """
        Whether the map will show a smaller position cursor (True), or no
        position cursor (False) when cursor is outside of map's range.

        Arguments
        - unlimited: tracking state
        """
        ...


    def isLocked(self) -> bool:
        """
        Gets whether the map is locked or not.
        
        A locked map may not be explored further.

        Returns
        - lock status
        """
        ...


    def setLocked(self, locked: bool) -> None:
        """
        Gets whether the map is locked or not.
        
        A locked map may not be explored further.

        Arguments
        - locked: status
        """
        ...


    class Scale(Enum):
        """
        An enum representing all possible scales a map can be set to.
        """

        CLOSEST = (0)
        CLOSE = (1)
        NORMAL = (2)
        FAR = (3)
        FARTHEST = (4)


        @staticmethod
        def valueOf(value: int) -> "Scale":
            """
            Get the scale given the raw value.

            Arguments
            - value: The raw scale

            Returns
            - The enum scale, or null for an invalid input

            Deprecated
            - Magic value
            """
            ...


        def getValue(self) -> int:
            """
            Get the raw value of this scale level.

            Returns
            - The scale value

            Deprecated
            - Magic value
            """
            ...



"""
Python module generated from Java source file com.google.common.base.CharMatcher

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import *
from java.util import Arrays
from java.util import BitSet
from typing import Any, Callable, Iterable, Tuple


class CharMatcher(Predicate):
    """
    Determines a True or False value for any Java `char` value, just as Predicate does
    for any Object. Also offers basic text processing methods based on this function.
    Implementations are strongly encouraged to be side-effect-free and immutable.
    
    Throughout the documentation of this class, the phrase "matching character" is used to mean
    "any `char` value `c` for which `this.matches(c)` returns `True`".
    
    **Warning:** This class deals only with `char` values, that is, <a
    href="http://www.unicode.org/glossary/#BMP_character">BMP characters</a>. It does not understand
    <a href="http://www.unicode.org/glossary/#supplementary_code_point">supplementary Unicode code
    points</a> in the range `0x10000` to `0x10FFFF` which includes the majority of
    assigned characters, including important CJK characters and emoji.
    
    Supplementary characters are <a
    href="https://docs.oracle.com/javase/8/docs/api/java/lang/Character.html#supplementary">encoded
    into a `String` using surrogate pairs</a>, and a `CharMatcher` treats these just as
    two separate characters. .countIn counts each supplementary character as 2 `char`s.
    
    For up-to-date Unicode character properties (digit, letter, etc.) and support for
    supplementary code points, use ICU4J UCharacter and UnicodeSet (freeze() after building). For
    basic text processing based on UnicodeSet use the ICU4J UnicodeSetSpanner.
    
    Example usages:
    
    ```
      String trimmed = .whitespace() whitespace()..trimFrom trimFrom(userInput);
      if (.ascii() ascii()..matchesAllOf matchesAllOf(s)) { ... }```
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/StringsExplained#charmatcher">`CharMatcher`
    </a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 1.0
    """

    @staticmethod
    def any() -> "CharMatcher":
        """
        Matches any character.

        Since
        - 19.0 (since 1.0 as constant `ANY`)
        """
        ...


    @staticmethod
    def none() -> "CharMatcher":
        """
        Matches no characters.

        Since
        - 19.0 (since 1.0 as constant `NONE`)
        """
        ...


    @staticmethod
    def whitespace() -> "CharMatcher":
        """
        Determines whether a character is whitespace according to the latest Unicode standard, as
        illustrated <a
        href="http://unicode.org/cldr/utility/list-unicodeset.jsp?a=%5Cp%7Bwhitespace%7D">here</a>.
        This is not the same definition used by other Java APIs. (See a <a
        href="https://docs.google.com/spreadsheets/d/1kq4ECwPjHX9B8QUCTPclgsDCXYaj7T-FlT4tB5q3ahk/edit">comparison
        of several definitions of "whitespace"</a>.)
        
        All Unicode White_Space characters are on the BMP and thus supported by this API.
        
        **Note:** as the Unicode definition evolves, we will modify this matcher to keep it up to
        date.

        Since
        - 19.0 (since 1.0 as constant `WHITESPACE`)
        """
        ...


    @staticmethod
    def breakingWhitespace() -> "CharMatcher":
        """
        Determines whether a character is a breaking whitespace (that is, a whitespace which can be
        interpreted as a break between words for formatting purposes). See .whitespace() for a
        discussion of that term.

        Since
        - 19.0 (since 2.0 as constant `BREAKING_WHITESPACE`)
        """
        ...


    @staticmethod
    def ascii() -> "CharMatcher":
        """
        Determines whether a character is ASCII, meaning that its code point is less than 128.

        Since
        - 19.0 (since 1.0 as constant `ASCII`)
        """
        ...


    @staticmethod
    def digit() -> "CharMatcher":
        """
        Determines whether a character is a BMP digit according to <a
        href="http://unicode.org/cldr/utility/list-unicodeset.jsp?a=%5Cp%7Bdigit%7D">Unicode</a>. If
        you only care to match ASCII digits, you can use `inRange('0', '9')`.

        Since
        - 19.0 (since 1.0 as constant `DIGIT`)

        Deprecated
        - Many digits are supplementary characters; see the class documentation.
        """
        ...


    @staticmethod
    def javaDigit() -> "CharMatcher":
        """
        Determines whether a character is a BMP digit according to Character.isDigit(char)
        Java's definition. If you only care to match ASCII digits, you can use `inRange('0',
        '9')`.

        Since
        - 19.0 (since 1.0 as constant `JAVA_DIGIT`)

        Deprecated
        - Many digits are supplementary characters; see the class documentation.
        """
        ...


    @staticmethod
    def javaLetter() -> "CharMatcher":
        """
        Determines whether a character is a BMP letter according to Character.isLetter(char) Java's definition. If you only care to match letters of the Latin
        alphabet, you can use `inRange('a', 'z').or(inRange('A', 'Z'))`.

        Since
        - 19.0 (since 1.0 as constant `JAVA_LETTER`)

        Deprecated
        - Most letters are supplementary characters; see the class documentation.
        """
        ...


    @staticmethod
    def javaLetterOrDigit() -> "CharMatcher":
        """
        Determines whether a character is a BMP letter or digit according to Character.isLetterOrDigit(char) Java's definition.

        Since
        - 19.0 (since 1.0 as constant `JAVA_LETTER_OR_DIGIT`).

        Deprecated
        - Most letters and digits are supplementary characters; see the class documentation.
        """
        ...


    @staticmethod
    def javaUpperCase() -> "CharMatcher":
        """
        Determines whether a BMP character is upper case according to Character.isUpperCase(char) Java's definition.

        Since
        - 19.0 (since 1.0 as constant `JAVA_UPPER_CASE`)

        Deprecated
        - Some uppercase characters are supplementary characters; see the class
            documentation.
        """
        ...


    @staticmethod
    def javaLowerCase() -> "CharMatcher":
        """
        Determines whether a BMP character is lower case according to Character.isLowerCase(char) Java's definition.

        Since
        - 19.0 (since 1.0 as constant `JAVA_LOWER_CASE`)

        Deprecated
        - Some lowercase characters are supplementary characters; see the class
            documentation.
        """
        ...


    @staticmethod
    def javaIsoControl() -> "CharMatcher":
        """
        Determines whether a character is an ISO control character as specified by Character.isISOControl(char).
        
        All ISO control codes are on the BMP and thus supported by this API.

        Since
        - 19.0 (since 1.0 as constant `JAVA_ISO_CONTROL`)
        """
        ...


    @staticmethod
    def invisible() -> "CharMatcher":
        """
        Determines whether a character is invisible; that is, if its Unicode category is any of
        SPACE_SEPARATOR, LINE_SEPARATOR, PARAGRAPH_SEPARATOR, CONTROL, FORMAT, SURROGATE, and
        PRIVATE_USE according to ICU4J.
        
        See also the Unicode Default_Ignorable_Code_Point property (available via ICU).

        Since
        - 19.0 (since 1.0 as constant `INVISIBLE`)

        Deprecated
        - Most invisible characters are supplementary characters; see the class
            documentation.
        """
        ...


    @staticmethod
    def singleWidth() -> "CharMatcher":
        """
        Determines whether a character is single-width (not double-width). When in doubt, this matcher
        errs on the side of returning `False` (that is, it tends to assume a character is
        double-width).
        
        **Note:** as the reference file evolves, we will modify this matcher to keep it up to
        date.
        
        See also <a href="http://www.unicode.org/reports/tr11/">UAX #11 East Asian Width</a>.

        Since
        - 19.0 (since 1.0 as constant `SINGLE_WIDTH`)

        Deprecated
        - Many such characters are supplementary characters; see the class documentation.
        """
        ...


    @staticmethod
    def is(match: str) -> "CharMatcher":
        """
        Returns a `char` matcher that matches only one specified BMP character.
        """
        ...


    @staticmethod
    def isNot(match: str) -> "CharMatcher":
        """
        Returns a `char` matcher that matches any character except the BMP character specified.
        
        To negate another `CharMatcher`, use .negate().
        """
        ...


    @staticmethod
    def anyOf(sequence: "CharSequence") -> "CharMatcher":
        """
        Returns a `char` matcher that matches any BMP character present in the given character
        sequence. Returns a bogus matcher if the sequence contains supplementary characters.
        """
        ...


    @staticmethod
    def noneOf(sequence: "CharSequence") -> "CharMatcher":
        """
        Returns a `char` matcher that matches any BMP character not present in the given
        character sequence. Returns a bogus matcher if the sequence contains supplementary characters.
        """
        ...


    @staticmethod
    def inRange(startInclusive: str, endInclusive: str) -> "CharMatcher":
        """
        Returns a `char` matcher that matches any character in a given BMP range (both endpoints
        are inclusive). For example, to match any lowercase letter of the English alphabet, use `CharMatcher.inRange('a', 'z')`.

        Raises
        - IllegalArgumentException: if `endInclusive < startInclusive`
        """
        ...


    @staticmethod
    def forPredicate(predicate: "Predicate"["Character"]) -> "CharMatcher":
        """
        Returns a matcher with identical behavior to the given Character-based predicate, but
        which operates on primitive `char` instances instead.
        """
        ...


    def matches(self, c: str) -> bool:
        """
        Determines a True or False value for the given character.
        """
        ...


    def negate(self) -> "CharMatcher":
        ...


    def and(self, other: "CharMatcher") -> "CharMatcher":
        """
        Returns a matcher that matches any character matched by both this matcher and `other`.
        """
        ...


    def or(self, other: "CharMatcher") -> "CharMatcher":
        """
        Returns a matcher that matches any character matched by either this matcher or `other`.
        """
        ...


    def precomputed(self) -> "CharMatcher":
        """
        Returns a `char` matcher functionally equivalent to this one, but which may be faster to
        query than the original; your mileage may vary. Precomputation takes time and is likely to be
        worthwhile only if the precomputed matcher is queried many thousands of times.
        
        This method has no effect (returns `this`) when called in GWT: it's unclear whether a
        precomputed matcher is faster, but it certainly consumes more memory, which doesn't seem like a
        worthwhile tradeoff in a browser.
        """
        ...


    def matchesAnyOf(self, sequence: "CharSequence") -> bool:
        """
        Returns `True` if a character sequence contains at least one matching BMP character.
        Equivalent to `!matchesNoneOf(sequence)`.
        
        The default implementation iterates over the sequence, invoking .matches for each
        character, until this returns `True` or the end is reached.

        Arguments
        - sequence: the character sequence to examine, possibly empty

        Returns
        - `True` if this matcher matches at least one character in the sequence

        Since
        - 8.0
        """
        ...


    def matchesAllOf(self, sequence: "CharSequence") -> bool:
        """
        Returns `True` if a character sequence contains only matching BMP characters.
        
        The default implementation iterates over the sequence, invoking .matches for each
        character, until this returns `False` or the end is reached.

        Arguments
        - sequence: the character sequence to examine, possibly empty

        Returns
        - `True` if this matcher matches every character in the sequence, including when
            the sequence is empty
        """
        ...


    def matchesNoneOf(self, sequence: "CharSequence") -> bool:
        """
        Returns `True` if a character sequence contains no matching BMP characters. Equivalent to
        `!matchesAnyOf(sequence)`.
        
        The default implementation iterates over the sequence, invoking .matches for each
        character, until this returns `True` or the end is reached.

        Arguments
        - sequence: the character sequence to examine, possibly empty

        Returns
        - `True` if this matcher matches no characters in the sequence, including when the
            sequence is empty
        """
        ...


    def indexIn(self, sequence: "CharSequence") -> int:
        """
        Returns the index of the first matching BMP character in a character sequence, or `-1` if
        no matching character is present.
        
        The default implementation iterates over the sequence in forward order calling .matches for each character.

        Arguments
        - sequence: the character sequence to examine from the beginning

        Returns
        - an index, or `-1` if no character matches
        """
        ...


    def indexIn(self, sequence: "CharSequence", start: int) -> int:
        """
        Returns the index of the first matching BMP character in a character sequence, starting from a
        given position, or `-1` if no character matches after that position.
        
        The default implementation iterates over the sequence in forward order, beginning at `start`, calling .matches for each character.

        Arguments
        - sequence: the character sequence to examine
        - start: the first index to examine; must be nonnegative and no greater than `sequence.length()`

        Returns
        - the index of the first matching character, guaranteed to be no less than `start`,
            or `-1` if no character matches

        Raises
        - IndexOutOfBoundsException: if start is negative or greater than `sequence.length()`
        """
        ...


    def lastIndexIn(self, sequence: "CharSequence") -> int:
        """
        Returns the index of the last matching BMP character in a character sequence, or `-1` if
        no matching character is present.
        
        The default implementation iterates over the sequence in reverse order calling .matches for each character.

        Arguments
        - sequence: the character sequence to examine from the end

        Returns
        - an index, or `-1` if no character matches
        """
        ...


    def countIn(self, sequence: "CharSequence") -> int:
        """
        Returns the number of matching `char`s found in a character sequence.
        
        Counts 2 per supplementary character, such as for .whitespace()..negate().
        """
        ...


    def removeFrom(self, sequence: "CharSequence") -> str:
        """
        Returns a string containing all non-matching characters of a character sequence, in order. For
        example:
        
        ````CharMatcher.is('a').removeFrom("bazaar")````
        
        ... returns `"bzr"`.
        """
        ...


    def retainFrom(self, sequence: "CharSequence") -> str:
        """
        Returns a string containing all matching BMP characters of a character sequence, in order. For
        example:
        
        ````CharMatcher.is('a').retainFrom("bazaar")````
        
        ... returns `"aaa"`.
        """
        ...


    def replaceFrom(self, sequence: "CharSequence", replacement: str) -> str:
        """
        Returns a string copy of the input character sequence, with each matching BMP character
        replaced by a given replacement character. For example:
        
        ````CharMatcher.is('a').replaceFrom("radar", 'o')````
        
        ... returns `"rodor"`.
        
        The default implementation uses .indexIn(CharSequence) to find the first matching
        character, then iterates the remainder of the sequence calling .matches(char) for each
        character.

        Arguments
        - sequence: the character sequence to replace matching characters in
        - replacement: the character to append to the result string in place of each matching
            character in `sequence`

        Returns
        - the new string
        """
        ...


    def replaceFrom(self, sequence: "CharSequence", replacement: "CharSequence") -> str:
        """
        Returns a string copy of the input character sequence, with each matching BMP character
        replaced by a given replacement sequence. For example:
        
        ````CharMatcher.is('a').replaceFrom("yaha", "oo")````
        
        ... returns `"yoohoo"`.
        
        **Note:** If the replacement is a fixed string with only one character, you are better
        off calling .replaceFrom(CharSequence, char) directly.

        Arguments
        - sequence: the character sequence to replace matching characters in
        - replacement: the characters to append to the result string in place of each matching
            character in `sequence`

        Returns
        - the new string
        """
        ...


    def trimFrom(self, sequence: "CharSequence") -> str:
        """
        Returns a substring of the input character sequence that omits all matching BMP characters from
        the beginning and from the end of the string. For example:
        
        ````CharMatcher.anyOf("ab").trimFrom("abacatbab")````
        
        ... returns `"cat"`.
        
        Note that:
        
        ````CharMatcher.inRange('\0', ' ').trimFrom(str)````
        
        ... is equivalent to String.trim().
        """
        ...


    def trimLeadingFrom(self, sequence: "CharSequence") -> str:
        """
        Returns a substring of the input character sequence that omits all matching BMP characters from
        the beginning of the string. For example:
        
        ````CharMatcher.anyOf("ab").trimLeadingFrom("abacatbab")````
        
        ... returns `"catbab"`.
        """
        ...


    def trimTrailingFrom(self, sequence: "CharSequence") -> str:
        """
        Returns a substring of the input character sequence that omits all matching BMP characters from
        the end of the string. For example:
        
        ````CharMatcher.anyOf("ab").trimTrailingFrom("abacatbab")````
        
        ... returns `"abacat"`.
        """
        ...


    def collapseFrom(self, sequence: "CharSequence", replacement: str) -> str:
        """
        Returns a string copy of the input character sequence, with each group of consecutive matching
        BMP characters replaced by a single replacement character. For example:
        
        ````CharMatcher.anyOf("eko").collapseFrom("bookkeeper", '-')````
        
        ... returns `"b-p-r"`.
        
        The default implementation uses .indexIn(CharSequence) to find the first matching
        character, then iterates the remainder of the sequence calling .matches(char) for each
        character.

        Arguments
        - sequence: the character sequence to replace matching groups of characters in
        - replacement: the character to append to the result string in place of each group of
            matching characters in `sequence`

        Returns
        - the new string
        """
        ...


    def trimAndCollapseFrom(self, sequence: "CharSequence", replacement: str) -> str:
        """
        Collapses groups of matching characters exactly as .collapseFrom does, except that
        groups of matching BMP characters at the start or end of the sequence are removed without
        replacement.
        """
        ...


    def apply(self, character: "Character") -> bool:
        """
        Deprecated
        - Provided only to satisfy the Predicate interface; use .matches
            instead.
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this `CharMatcher`, such as `CharMatcher.or(WHITESPACE, JAVA_DIGIT)`.
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.BloomFilterStrategies

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.hash import *
from com.google.common.math import LongMath
from com.google.common.primitives import Ints
from com.google.common.primitives import Longs
from enum import Enum
from java.math import RoundingMode
from java.util import Arrays
from java.util.concurrent.atomic import AtomicLongArray
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class BloomFilterStrategies(Enum):
    """
    Collections of strategies of generating the k * log(M) bits required for an element to be mapped
    to a BloomFilter of M bits and k hash functions. These strategies are part of the serialized form
    of the Bloom filters that use them, thus they must be preserved as is (no updates allowed, only
    introduction of new versions).
    
    Important: the order of the constants cannot change, and they cannot be deleted - we depend on
    their ordinal for BloomFilter serialization.

    Author(s)
    - Kurt Alfred Kluever
    """

    MURMUR128_MITZ_32 = 0
    """
    See "Less Hashing, Same Performance: Building a Better Bloom Filter" by Adam Kirsch and Michael
    Mitzenmacher. The paper argues that this trick doesn't significantly deteriorate the
    performance of a Bloom filter (yet only needs two 32bit hash functions).
    """
    MURMUR128_MITZ_64 = 1
    """
    This strategy uses all 128 bits of Hashing.murmur3_128 when hashing. It looks different
    from the implementation in MURMUR128_MITZ_32 because we're avoiding the multiplication in the
    loop and doing a (much simpler) += hash2. We're also changing the index to a positive number by
    AND'ing with Long.MAX_VALUE instead of flipping the bits.
    """



"""
Python module generated from Java source file org.bukkit.inventory.meta.components.UseCooldownComponent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import NamespacedKey
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.inventory.meta.components import *
from typing import Any, Callable, Iterable, Tuple


class UseCooldownComponent(ConfigurationSerializable):
    """
    Represents a component which determines the cooldown applied to use of this
    item.
    """

    def getCooldownSeconds(self) -> float:
        """
        Gets the time in seconds it will take for an item in this cooldown group
        to be available to use again.

        Returns
        - cooldown time
        """
        ...


    def setCooldownSeconds(self, cooldown: float) -> None:
        """
        Sets the time in seconds it will take for an item in this cooldown group
        to be available to use again.

        Arguments
        - cooldown: new eat time, must be greater than 0
        """
        ...


    def getCooldownGroup(self) -> "NamespacedKey":
        """
        Gets the custom cooldown group to be used for similar items, if set.

        Returns
        - the cooldown group
        """
        ...


    def setCooldownGroup(self, song: "NamespacedKey") -> None:
        """
        Sets the custom cooldown group to be used for similar items.

        Arguments
        - song: the cooldown group
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerBucketEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Bukkit
from org.bukkit import Material
from org.bukkit.block import Block
from org.bukkit.block import BlockFace
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event.player import *
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemStack
from org.bukkit.material import MaterialData
from typing import Any, Callable, Iterable, Tuple


class PlayerBucketEvent(PlayerEvent, Cancellable):
    """
    Called when a player interacts with a Bucket
    """

    def __init__(self, who: "Player", blockClicked: "Block", blockFace: "BlockFace", bucket: "Material", itemInHand: "ItemStack"):
        ...


    def __init__(self, who: "Player", block: "Block", blockClicked: "Block", blockFace: "BlockFace", bucket: "Material", itemInHand: "ItemStack"):
        ...


    def __init__(self, who: "Player", block: "Block", blockClicked: "Block", blockFace: "BlockFace", bucket: "Material", itemInHand: "ItemStack", hand: "EquipmentSlot"):
        ...


    def getBucket(self) -> "Material":
        """
        Returns the bucket used in this event

        Returns
        - the used bucket
        """
        ...


    def getItemStack(self) -> "ItemStack":
        """
        Get the resulting item in hand after the bucket event

        Returns
        - ItemStack hold in hand after the event.
        """
        ...


    def setItemStack(self, itemStack: "ItemStack") -> None:
        """
        Set the item in hand after the event

        Arguments
        - itemStack: the new held ItemStack after the bucket event.
        """
        ...


    def getBlock(self) -> "Block":
        """
        Gets the block involved in this event.

        Returns
        - The Block which block is involved in this event
        """
        ...


    def getBlockClicked(self) -> "Block":
        """
        Return the block clicked

        Returns
        - the clicked block
        """
        ...


    def getBlockFace(self) -> "BlockFace":
        """
        Get the face on the clicked block

        Returns
        - the clicked face
        """
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Get the hand that was used in this event.

        Returns
        - the hand
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...



"""
Python module generated from Java source file com.google.common.net.InternetDomainName

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Ascii
from com.google.common.base import CharMatcher
from com.google.common.base import Joiner
from com.google.common.base import Optional
from com.google.common.base import Splitter
from com.google.common.collect import ImmutableList
from com.google.common.net import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import Immutable
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.thirdparty.publicsuffix import PublicSuffixPatterns
from com.google.thirdparty.publicsuffix import PublicSuffixType
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class InternetDomainName:
    """
    An immutable well-formed internet domain name, such as `com` or `foo.co.uk`. Only
    syntactic analysis is performed; no DNS lookups or other network interactions take place. Thus
    there is no guarantee that the domain actually exists on the internet.
    
    One common use of this class is to determine whether a given string is likely to represent an
    addressable domain on the web -- that is, for a candidate string `"xxx"`, might browsing to
    `"http://xxx/"` result in a webpage being displayed? In the past, this test was frequently
    done by determining whether the domain ended with a .isPublicSuffix() public suffix
    but was not itself a public suffix. However, this test is no longer accurate. There are many
    domains which are both public suffixes and addressable as hosts; `"uk.com"` is one example.
    Using the subset of public suffixes that are .isRegistrySuffix() registry suffixes,
    one can get a better result, as only a few registry suffixes are addressable. However, the most
    useful test to determine if a domain is a plausible web host is .hasPublicSuffix(). This
    will return `True` for many domains which (currently) are not hosts, such as `"com"`,
    but given that any public suffix may become a host without warning, it is better to err on the
    side of permissiveness and thus avoid spurious rejection of valid sites. Of course, to actually
    determine addressability of any host, clients of this class will need to perform their own DNS
    lookups.
    
    During construction, names are normalized in two ways:
    
    <ol>
      - ASCII uppercase characters are converted to lowercase.
      - Unicode dot separators other than the ASCII period (`'.'`) are converted to the ASCII
          period.
    </ol>
    
    The normalized values will be returned from .toString() and .parts(), and will
    be reflected in the result of .equals(Object).
    
    <a href="http://en.wikipedia.org/wiki/Internationalized_domain_name">Internationalized domain
    names</a> such as `网络.cn` are supported, as are the equivalent <a
    href="http://en.wikipedia.org/wiki/Internationalized_domain_name">IDNA Punycode-encoded</a>
    versions.

    Author(s)
    - Catherine Berry

    Since
    - 5.0
    """

    @staticmethod
    def from(domain: str) -> "InternetDomainName":
        """
        Returns an instance of InternetDomainName after lenient validation. Specifically,
        validation against <a href="http://www.ietf.org/rfc/rfc3490.txt">RFC 3490</a>
        ("Internationalizing Domain Names in Applications") is skipped, while validation against <a
        href="http://www.ietf.org/rfc/rfc1035.txt">RFC 1035</a> is relaxed in the following ways:
        
        
          - Any part containing non-ASCII characters is considered valid.
          - Underscores ('_') are permitted wherever dashes ('-') are permitted.
          - Parts other than the final part may start with a digit, as mandated by <a
              href="https://tools.ietf.org/html/rfc1123#section-2">RFC 1123</a>.

        Arguments
        - domain: A domain name (not IP address)

        Raises
        - IllegalArgumentException: if `domain` is not syntactically valid according to
            .isValid

        Since
        - 10.0 (previously named `fromLenient`)
        """
        ...


    def parts(self) -> "ImmutableList"[str]:
        """
        Returns the individual components of this domain name, normalized to all lower case. For
        example, for the domain name `mail.google.com`, this method returns the list `["mail", "google", "com"]`.
        """
        ...


    def isPublicSuffix(self) -> bool:
        """
        Indicates whether this domain name represents a *public suffix*, as defined by the Mozilla
        Foundation's <a href="http://publicsuffix.org/">Public Suffix List</a> (PSL). A public suffix
        is one under which Internet users can directly register names, such as `com`, `co.uk` or `pvt.k12.wy.us`. Examples of domain names that are *not* public suffixes
        include `google.com`, `foo.co.uk`, and `myblog.blogspot.com`.
        
        Public suffixes are a proper superset of .isRegistrySuffix() registry suffixes.
        The list of public suffixes additionally contains privately owned domain names under which
        Internet users can register subdomains. An example of a public suffix that is not a registry
        suffix is `blogspot.com`. Note that it is True that all public suffixes *have*
        registry suffixes, since domain name registries collectively control all internet domain names.
        
        For considerations on whether the public suffix or registry suffix designation is more
        suitable for your application, see <a
        href="https://github.com/google/guava/wiki/InternetDomainNameExplained">this article</a>.

        Returns
        - `True` if this domain name appears exactly on the public suffix list

        Since
        - 6.0
        """
        ...


    def hasPublicSuffix(self) -> bool:
        """
        Indicates whether this domain name ends in a .isPublicSuffix() public suffix,
        including if it is a public suffix itself. For example, returns `True` for `www.google.com`, `foo.co.uk` and `com`, but not for `invalid` or `google.invalid`. This is the recommended method for determining whether a domain is potentially
        an addressable host.
        
        Note that this method is equivalent to .hasRegistrySuffix() because all registry
        suffixes are public suffixes *and* all public suffixes have registry suffixes.

        Since
        - 6.0
        """
        ...


    def publicSuffix(self) -> "InternetDomainName":
        """
        Returns the .isPublicSuffix() public suffix portion of the domain name, or `null` if no public suffix is present.

        Since
        - 6.0
        """
        ...


    def isUnderPublicSuffix(self) -> bool:
        """
        Indicates whether this domain name ends in a .isPublicSuffix() public suffix,
        while not being a public suffix itself. For example, returns `True` for `www.google.com`, `foo.co.uk` and `myblog.blogspot.com`, but not for `com`,
        `co.uk`, `google.invalid`, or `blogspot.com`.
        
        This method can be used to determine whether it will probably be possible to set cookies on
        the domain, though even that depends on individual browsers' implementations of cookie
        controls. See <a href="http://www.ietf.org/rfc/rfc2109.txt">RFC 2109</a> for details.

        Since
        - 6.0
        """
        ...


    def isTopPrivateDomain(self) -> bool:
        """
        Indicates whether this domain name is composed of exactly one subdomain component followed by a
        .isPublicSuffix() public suffix. For example, returns `True` for `google.com` `foo.co.uk`, and `myblog.blogspot.com`, but not for `www.google.com`, `co.uk`, or `blogspot.com`.
        
        This method can be used to determine whether a domain is probably the highest level for
        which cookies may be set, though even that depends on individual browsers' implementations of
        cookie controls. See <a href="http://www.ietf.org/rfc/rfc2109.txt">RFC 2109</a> for details.

        Since
        - 6.0
        """
        ...


    def topPrivateDomain(self) -> "InternetDomainName":
        """
        Returns the portion of this domain name that is one level beneath the .isPublicSuffix() public suffix. For example, for `x.adwords.google.co.uk` it returns
        `google.co.uk`, since `co.uk` is a public suffix. Similarly, for `myblog.blogspot.com` it returns the same domain, `myblog.blogspot.com`, since `blogspot.com` is a public suffix.
        
        If .isTopPrivateDomain() is True, the current domain name instance is returned.
        
        This method can be used to determine the probable highest level parent domain for which
        cookies may be set, though even that depends on individual browsers' implementations of cookie
        controls.

        Raises
        - IllegalStateException: if this domain does not end with a public suffix

        Since
        - 6.0
        """
        ...


    def isRegistrySuffix(self) -> bool:
        """
        Indicates whether this domain name represents a *registry suffix*, as defined by a subset
        of the Mozilla Foundation's <a href="http://publicsuffix.org/">Public Suffix List</a> (PSL). A
        registry suffix is one under which Internet users can directly register names via a domain name
        registrar, and have such registrations lawfully protected by internet-governing bodies such as
        ICANN. Examples of registry suffixes include `com`, `co.uk`, and `pvt.k12.wy.us`. Examples of domain names that are *not* registry suffixes include `google.com` and `foo.co.uk`.
        
        Registry suffixes are a proper subset of .isPublicSuffix() public suffixes. The
        list of public suffixes additionally contains privately owned domain names under which Internet
        users can register subdomains. An example of a public suffix that is not a registry suffix is
        `blogspot.com`. Note that it is True that all public suffixes *have* registry
        suffixes, since domain name registries collectively control all internet domain names.
        
        For considerations on whether the public suffix or registry suffix designation is more
        suitable for your application, see <a
        href="https://github.com/google/guava/wiki/InternetDomainNameExplained">this article</a>.

        Returns
        - `True` if this domain name appears exactly on the public suffix list as part of
            the registry suffix section (labelled "ICANN").

        Since
        - 23.3
        """
        ...


    def hasRegistrySuffix(self) -> bool:
        """
        Indicates whether this domain name ends in a .isRegistrySuffix() registry suffix,
        including if it is a registry suffix itself. For example, returns `True` for `www.google.com`, `foo.co.uk` and `com`, but not for `invalid` or `google.invalid`.
        
        Note that this method is equivalent to .hasPublicSuffix() because all registry
        suffixes are public suffixes *and* all public suffixes have registry suffixes.

        Since
        - 23.3
        """
        ...


    def registrySuffix(self) -> "InternetDomainName":
        """
        Returns the .isRegistrySuffix() registry suffix portion of the domain name, or
        `null` if no registry suffix is present.

        Since
        - 23.3
        """
        ...


    def isUnderRegistrySuffix(self) -> bool:
        """
        Indicates whether this domain name ends in a .isRegistrySuffix() registry suffix,
        while not being a registry suffix itself. For example, returns `True` for `www.google.com`, `foo.co.uk` and `blogspot.com`, but not for `com`, `co.uk`, or `google.invalid`.

        Since
        - 23.3
        """
        ...


    def isTopDomainUnderRegistrySuffix(self) -> bool:
        """
        Indicates whether this domain name is composed of exactly one subdomain component followed by a
        .isRegistrySuffix() registry suffix. For example, returns `True` for `google.com`, `foo.co.uk`, and `blogspot.com`, but not for `www.google.com`,
        `co.uk`, or `myblog.blogspot.com`.
        
        **Warning:** This method should not be used to determine the probable highest level
        parent domain for which cookies may be set. Use .topPrivateDomain() for that purpose.

        Since
        - 23.3
        """
        ...


    def topDomainUnderRegistrySuffix(self) -> "InternetDomainName":
        """
        Returns the portion of this domain name that is one level beneath the .isRegistrySuffix() registry suffix. For example, for `x.adwords.google.co.uk` it
        returns `google.co.uk`, since `co.uk` is a registry suffix. Similarly, for `myblog.blogspot.com` it returns `blogspot.com`, since `com` is a registry suffix.
        
        If .isTopDomainUnderRegistrySuffix() is True, the current domain name instance is
        returned.
        
        **Warning:** This method should not be used to determine whether a domain is probably the
        highest level for which cookies may be set. Use .isTopPrivateDomain() for that purpose.

        Raises
        - IllegalStateException: if this domain does not end with a registry suffix

        Since
        - 23.3
        """
        ...


    def hasParent(self) -> bool:
        """
        Indicates whether this domain is composed of two or more parts.
        """
        ...


    def parent(self) -> "InternetDomainName":
        """
        Returns an `InternetDomainName` that is the immediate ancestor of this one; that is, the
        current domain with the leftmost part removed. For example, the parent of `www.google.com` is `google.com`.

        Raises
        - IllegalStateException: if the domain has no parent, as determined by .hasParent
        """
        ...


    def child(self, leftParts: str) -> "InternetDomainName":
        """
        Creates and returns a new `InternetDomainName` by prepending the argument and a dot to
        the current name. For example, `InternetDomainName.from("foo.com").child("www.bar")`
        returns a new `InternetDomainName` with the value `www.bar.foo.com`. Only lenient
        validation is performed, as described .from(String) here.

        Raises
        - NullPointerException: if leftParts is null
        - IllegalArgumentException: if the resulting name is not valid
        """
        ...


    @staticmethod
    def isValid(name: str) -> bool:
        """
        Indicates whether the argument is a syntactically valid domain name using lenient validation.
        Specifically, validation against <a href="http://www.ietf.org/rfc/rfc3490.txt">RFC 3490</a>
        ("Internationalizing Domain Names in Applications") is skipped.
        
        The following two code snippets are equivalent:
        
        ````domainName = InternetDomainName.isValid(name)
            ? InternetDomainName.from(name)
            : DEFAULT_DOMAIN;````
        
        ````try {
          domainName = InternetDomainName.from(name);` catch (IllegalArgumentException e) {
          domainName = DEFAULT_DOMAIN;
        }
        }```

        Since
        - 8.0 (previously named `isValidLenient`)
        """
        ...


    def toString(self) -> str:
        """
        Returns the domain name, normalized to all lower case.
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Equality testing is based on the text supplied by the caller, after normalization as described
        in the class documentation. For example, a non-ASCII Unicode domain name and the Punycode
        version of the same domain name would not be considered equal.
        """
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file com.google.common.cache.ForwardingCache

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Preconditions
from com.google.common.cache import *
from com.google.common.collect import ForwardingObject
from com.google.common.collect import ImmutableMap
from java.util.concurrent import Callable
from java.util.concurrent import ConcurrentMap
from java.util.concurrent import ExecutionException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ForwardingCache(ForwardingObject, Cache):
    """
    A cache which forwards all its method calls to another cache. Subclasses should override one or
    more methods to modify the behavior of the backing cache as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.

    Author(s)
    - Charles Fry

    Since
    - 10.0
    """

    def getIfPresent(self, key: "Object") -> "V":
        """
        Since
        - 11.0
        """
        ...


    def get(self, key: "K", valueLoader: "Callable"["V"]) -> "V":
        """
        Since
        - 11.0
        """
        ...


    def getAllPresent(self, keys: Iterable["Object"]) -> "ImmutableMap"["K", "V"]:
        """
        Since
        - 11.0
        """
        ...


    def put(self, key: "K", value: "V") -> None:
        """
        Since
        - 11.0
        """
        ...


    def putAll(self, m: dict["K", "V"]) -> None:
        """
        Since
        - 12.0
        """
        ...


    def invalidate(self, key: "Object") -> None:
        ...


    def invalidateAll(self, keys: Iterable["Object"]) -> None:
        """
        Since
        - 11.0
        """
        ...


    def invalidateAll(self) -> None:
        ...


    def size(self) -> int:
        ...


    def stats(self) -> "CacheStats":
        ...


    def asMap(self) -> "ConcurrentMap"["K", "V"]:
        ...


    def cleanUp(self) -> None:
        ...


    class SimpleForwardingCache(ForwardingCache):
        """
        A simplified version of ForwardingCache where subclasses can pass in an already
        constructed Cache as the delegate.

        Since
        - 10.0
        """





"""
Python module generated from Java source file org.joml.sampling.SpiralSampling

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import Random
from org.joml.sampling import *
from typing import Any, Callable, Iterable, Tuple


class SpiralSampling:
    """
    Creates samples on a spiral around a center point.

    Author(s)
    - Kai Burjack
    """

    def __init__(self, seed: int):
        """
        Create a new instance of SpiralSampling and initialize the random number generator with the given `seed`.

        Arguments
        - seed: the seed to initialize the random number generator with
        """
        ...


    def createEquiAngle(self, radius: float, numRotations: int, numSamples: int, callback: "Callback2d") -> None:
        """
        Create `numSamples` number of samples on a spiral with maximum radius `radius` around the center using `numRotations` number of rotations
        along the spiral, and call the given `callback` for each sample generated.
        
        The generated sample points are distributed with equal angle differences around the spiral, so they concentrate towards the center.

        Arguments
        - radius: the maximum radius of the spiral
        - numRotations: the number of rotations of the spiral
        - numSamples: the number of samples to generate
        - callback: will be called for each sample generated
        """
        ...


    def createEquiAngle(self, radius: float, numRotations: int, numSamples: int, jitter: float, callback: "Callback2d") -> None:
        """
        Create `numSamples` number of samples on a spiral with maximum radius `radius` around the center using `numRotations` number of rotations
        along the spiral, and call the given `callback` for each sample generated.
        
        The generated sample points are distributed with equal angle differences around the spiral, so they concentrate towards the center.
        
        Additionally, the radius of each sample point is jittered by the given `jitter` factor.

        Arguments
        - radius: the maximum radius of the spiral
        - numRotations: the number of rotations of the spiral
        - numSamples: the number of samples to generate
        - jitter: the factor by which the radius of each sample point is jittered. Possible values are `[0..1]`
        - callback: will be called for each sample generated
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Comparator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import Directional
from org.bukkit.block.data import Powerable
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Comparator(Directional, Powerable):
    """
    'mode' indicates what mode this comparator will operate in.
    """

    def getMode(self) -> "Mode":
        """
        Gets the value of the 'mode' property.

        Returns
        - the 'mode' value
        """
        ...


    def setMode(self, mode: "Mode") -> None:
        """
        Sets the value of the 'mode' property.

        Arguments
        - mode: the new 'mode' value
        """
        ...


    class Mode(Enum):
        """
        The mode in which a comparator will operate in.
        """

        COMPARE = 0
        """
        The default mode, similar to a transistor. The comparator will turn
        off if either side input is greater than the rear input.
        """
        SUBTRACT = 1
        """
        Alternate subtraction mode. The output signal strength will be equal
        to max(rear-max(left,right),0).
        """



"""
Python module generated from Java source file org.bukkit.event.inventory.CraftItemEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event.inventory import *
from org.bukkit.event.inventory.InventoryType import SlotType
from org.bukkit.inventory import CraftingInventory
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import Recipe
from typing import Any, Callable, Iterable, Tuple


class CraftItemEvent(InventoryClickEvent):
    """
    Called when the recipe of an Item is completed inside a crafting matrix.
    """

    def __init__(self, recipe: "Recipe", what: "InventoryView", type: "SlotType", slot: int, click: "ClickType", action: "InventoryAction"):
        ...


    def __init__(self, recipe: "Recipe", what: "InventoryView", type: "SlotType", slot: int, click: "ClickType", action: "InventoryAction", key: int):
        ...


    def getRecipe(self) -> "Recipe":
        """
        Returns
        - A copy of the current recipe on the crafting matrix.
        """
        ...


    def getInventory(self) -> "CraftingInventory":
        ...



"""
Python module generated from Java source file org.joml.sampling.Callback2d

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml.sampling import *
from typing import Any, Callable, Iterable, Tuple


class Callback2d:
    """
    Callback used for notifying about a new generated 2D sample.

    Author(s)
    - Kai Burjack
    """

    def onNewSample(self, x: float, y: float) -> None:
        """
        Will be called whenever a new sample with the given coordinates `(x, y)` is generated.

        Arguments
        - x: the x coordinate of the new sample point
        - y: the y coordinate of the new sample point
        """
        ...
